/*
 * Copyright (C) 1997-2005 Andre Noll <maan@systemlinux.org>
 * 
 *     This program is free software; you can redistribute it and/or modify
 *     it under the terms of the GNU General Public License as published by
 *     the Free Software Foundation; either version 2 of the License, or
 *     (at your option) any later version.
 * 
 *     This program is distributed in the hope that it will be useful,
 *     but WITHOUT ANY WARRANTY; without even the implied warranty of
 *     MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *     GNU General Public License for more details.
 * 
 *     You should have received a copy of the GNU General Public License
 *     along with this program; if not, write to the Free Software
 *     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111, USA.
 */

#include "para.h"
#include "server.cmdline.h"

/* status flags */
#define AFS_NOMORE 1
#define AFS_NEXT 2
#define AFS_REPOS 4
#define AFS_PLAYING 8
#define DBT_CHANGE 16

#define MMD_INFO_SIZE 16384

/* data needed to authenticate user */
struct user{
	char name[MAXLINE];
	char pubkey_file[_POSIX_PATH_MAX];
	unsigned int perms;
};

/* contains data for arbitrary command */
struct command {
	char *name;
	unsigned int perms;
	char *description;
	char *usage;
	char *help;
	int (*handler)(int, int, char **);
};

struct audio_format {
	struct timeval chunk_tv; /* length of one chunk of data */
	struct timeval eof_tv; /* timeout on eof */
	const char *name;
	int (*init)(void*);
	char *(*get_header_info)(int *);
	int (*get_file_info)(FILE*, char*, long unsigned*, int*);
	/*
	 * must return < 0 on errors, == 0 on eof, > 0 ok. If ret > 0, store
	 * buflen in given pointer. If that len is positive, *bufptr is not NULL and
	 * contains said chunk.
	*/
	void (*close_audio_file)(FILE *f);
	int (*reposition_stream)(long unsigned request);
	int (*read_chunk)(long unsigned chunk_num, size_t *len, char **bufptr);
};

struct dbtool {
	const char *name;
	struct command *cmd_list;
	char **(*get_song_list)(int);
	void  (*update_song)(char *);
	int (*init)(struct dbtool *);
	void (*shutdown)(void);
	const char **err_list;
};

struct sender_command_data{
	int cmd_num; /* >= 0 indicates that a server cmd is already queued */
	int sender_num;
	struct sockaddr_in self;
	struct in_addr addr; /* allow/deny, add/remove */
	int netmask; /* allow/deny */
	int port; /*tcp or udp */
};

/* there's one struct of this type which lives in shared memory
 * for communication between the server instances. Access to this
 * area is serialized via mmd_lock()/mmd_unlock(). There are two 
 * reasons for a variable to be included here:
 * (a) at least one command (i.e. child of the server) must be able to 
 *     change its value
 * or
 * (b) The contents are listed in the stat command and have to be up to date  
 */
struct misc_meta_data{
/* audio file info */ 
	long unsigned int size;
	char filename[_POSIX_PATH_MAX];
	time_t mtime;
/* afs status info */ 
	int audio_format;
	unsigned int afs_status_flags, new_afs_status_flags;
	long unsigned chunks_sent, chunks_total, repos_request, current_chunk;
	long offset; /* milliseconds */
	int seconds_total;
	struct timeval stream_start;
	char audio_file_info[MMD_INFO_SIZE];
/* server info */
	unsigned int events;
	unsigned int num_played;
	unsigned int num_commands;
	unsigned int num_connects;
	unsigned int active_connections;
	pid_t server_pid;
/* dbtool */
	char dbinfo[MMD_INFO_SIZE];
	int dbt_num; /* number of current dbtool */
	int dbt_change; /* requests dbtool change if non-negative */
/* sender */
	struct sender_command_data sender_cmd_data;
};

void afs_handle_sigchild(pid_t);
int afs_open_fifo(char *, int []);
int afs_init(struct gengetopt_args_info *);
int afs_send_chunk(void);
int afs_mainloop(struct timeval**);
char *afs_supported_audio_formats(void);
const char *audio_format_name(int);
int afs_handle_sighup(struct gengetopt_args_info *);

unsigned int afs_playing(void);
unsigned int afs_next(void);
unsigned int afs_repos(void);
unsigned int afs_paused(void);

int mysql_dbtool_init(struct dbtool*);
int dopey_dbtool_init(struct dbtool*);


int handle_connect(int fd, struct sockaddr_in *addr);

void mmd_unlock(void);
void mmd_lock(void);
